# encoding: UTF-8

module Author
  module PluginName
  
    class AppSpy < Sketchup::AppObserver

      @@attached = false unless defined?(@@attached)

      def self.attached?()
        @@attached ? true : false
      end

      def attach()
        detach() if @@attached
        Sketchup::add_observer(self)
      end

      def detach()
        Sketchup::remove_observer(self)
        @@attached = false
      end

      def initialize()
        @fco = nil
        @watched = []
        @@attached = attach()
        return self
      end

      def expectsStartupModelNotifications
        return true
      end

      def frameChange(fromPage, toPage, percent_done)
        # Actual code here.
      end

      def ignore_frames()
        if @fco
          Sketchup::Pages::remove_frame_change_observer(@fco)
          @fco = nil
        end
        @watched.keep_if {|model| model.valid? rescue false }
      end

      def ignore_model(model)
        @watched.delete(model)
        ignore_frames() if model == Sketchup.active_model
      end

      def watch_frames(model)
        ignore_frames() if @fco
        @fco = Sketchup::Pages::add_frame_change_observer(self)
        @watched << model unless @watched.include?(model)
      end

      def watching?(model)
        @watched.include?(model)
      end

      # AppObserver Callbacks here can call the above methods.

      def onActivateModel(model) # Mac only
        if watching?(model)
          watch_frames(model)
        else
          ignore_frames()
        end
      end

      def onNewModel(model)
        watch_frames(model)
      end

      def onOpenModel(model)
        watch_frames(model)
      end

      def onQuit()
        ignore_frames()
      end

      def onUnloadExtension(extension_name)
        ignore_frames()
      end

    end # AppSpy

    PluginSpy = AppSpy::new unless AppSpy.attached?
  
  end # extension module
end # toplevel module